@@ -34,6 +34,11 @@ USE_L10N_DEPRECATED_MSG = (
     "display numbers and dates using the format of the current locale."
 )
 
+CSRF_COOKIE_MASKED_DEPRECATED_MSG = (
+    "The CSRF_COOKIE_MASKED transitional setting is deprecated. Support for "
+    "it will be removed in Django 5.0."
+)
+
 
 class SettingsReference(str):
     """
@@ -83,9 +88,10 @@ class LazySettings(LazyObject):
 
     def __getattr__(self, name):
         """Return the value of a setting and cache it in self.__dict__."""
-        if self._wrapped is empty:
+        if (_wrapped := self._wrapped) is empty:
             self._setup(name)
-        val = getattr(self._wrapped, name)
+            _wrapped = self._wrapped
+        val = getattr(_wrapped, name)
 
         # Special case some settings which require further modification.
         # This is done here for performance reasons so the modified value is cached.
@@ -152,8 +158,9 @@ class LazySettings(LazyObject):
     def USE_L10N(self):
         stack = traceback.extract_stack()
         # Show a warning if the setting is used outside of Django.
-        # Stack index: -1 this line, -2 the caller.
-        filename, _, _, _ = stack[-2]
+        # Stack index: -1 this line, -2 the LazyObject __getattribute__(),
+        # -3 the caller.
+        filename, _, _, _ = stack[-3]
         if not filename.startswith(os.path.dirname(django.__file__)):
             warnings.warn(
                 USE_L10N_DEPRECATED_MSG,
@@ -187,6 +194,7 @@ class Settings:
             "INSTALLED_APPS",
             "TEMPLATE_DIRS",
             "LOCALE_PATHS",
+            "SECRET_KEY_FALLBACKS",
         )
         self._explicit_settings = set()
         for setting in dir(mod):
@@ -213,6 +221,9 @@ class Settings:
         if self.is_overridden("USE_DEPRECATED_PYTZ"):
             warnings.warn(USE_DEPRECATED_PYTZ_DEPRECATED_MSG, RemovedInDjango50Warning)
 
+        if self.is_overridden("CSRF_COOKIE_MASKED"):
+            warnings.warn(CSRF_COOKIE_MASKED_DEPRECATED_MSG, RemovedInDjango50Warning)
+
         if hasattr(time, "tzset") and self.TIME_ZONE:
             # When we can, attempt to validate the timezone. If we can't find
             # this file, no check happens and it's harmless.
@@ -262,6 +273,8 @@ class UserSettingsHolder:
         self._deleted.discard(name)
         if name == "USE_L10N":
             warnings.warn(USE_L10N_DEPRECATED_MSG, RemovedInDjango50Warning)
+        if name == "CSRF_COOKIE_MASKED":
+            warnings.warn(CSRF_COOKIE_MASKED_DEPRECATED_MSG, RemovedInDjango50Warning)
         super().__setattr__(name, value)
         if name == "USE_DEPRECATED_PYTZ":
             warnings.warn(USE_DEPRECATED_PYTZ_DEPRECATED_MSG, RemovedInDjango50Warning)
