@@ -1,6 +1,5 @@
 "File-based cache backend"
 import glob
-import hashlib
 import os
 import pickle
 import random
@@ -11,10 +10,11 @@ import zlib
 from django.core.cache.backends.base import DEFAULT_TIMEOUT, BaseCache
 from django.core.files import locks
 from django.core.files.move import file_move_safe
+from django.utils.crypto import md5
 
 
 class FileBasedCache(BaseCache):
-    cache_suffix = '.djcache'
+    cache_suffix = ".djcache"
     pickle_protocol = pickle.HIGHEST_PROTOCOL
 
     def __init__(self, dir, params):
@@ -31,7 +31,7 @@ class FileBasedCache(BaseCache):
     def get(self, key, default=None, version=None):
         fname = self._key_to_file(key, version)
         try:
-            with open(fname, 'rb') as f:
+            with open(fname, "rb") as f:
                 if not self._is_expired(f):
                     return pickle.loads(zlib.decompress(f.read()))
         except FileNotFoundError:
@@ -50,7 +50,7 @@ class FileBasedCache(BaseCache):
         fd, tmp_path = tempfile.mkstemp(dir=self._dir)
         renamed = False
         try:
-            with open(fd, 'wb') as f:
+            with open(fd, "wb") as f:
                 self._write_content(f, timeout, value)
             file_move_safe(tmp_path, fname, allow_overwrite=True)
             renamed = True
@@ -60,7 +60,7 @@ class FileBasedCache(BaseCache):
 
     def touch(self, key, timeout=DEFAULT_TIMEOUT, version=None):
         try:
-            with open(self._key_to_file(key, version), 'r+b') as f:
+            with open(self._key_to_file(key, version), "r+b") as f:
                 try:
                     locks.lock(f, locks.LOCK_EX)
                     if self._is_expired(f):
@@ -91,7 +91,7 @@ class FileBasedCache(BaseCache):
     def has_key(self, key, version=None):
         fname = self._key_to_file(key, version)
         if os.path.exists(fname):
-            with open(fname, 'rb') as f:
+            with open(fname, "rb") as f:
                 return not self._is_expired(f)
         return False
 
@@ -108,23 +108,34 @@ class FileBasedCache(BaseCache):
         if self._cull_frequency == 0:
             return self.clear()  # Clear the cache when CULL_FREQUENCY = 0
         # Delete a random selection of entries
-        filelist = random.sample(filelist,
-                                 int(num_entries / self._cull_frequency))
+        filelist = random.sample(filelist, int(num_entries / self._cull_frequency))
         for fname in filelist:
             self._delete(fname)
 
     def _createdir(self):
-        os.makedirs(self._dir, 0o700, exist_ok=True)
+        # Set the umask because os.makedirs() doesn't apply the "mode" argument
+        # to intermediate-level directories.
+        old_umask = os.umask(0o077)
+        try:
+            os.makedirs(self._dir, 0o700, exist_ok=True)
+        finally:
+            os.umask(old_umask)
 
     def _key_to_file(self, key, version=None):
         """
         Convert a key into a cache file path. Basically this is the
         root cache path joined with the md5sum of the key and a suffix.
         """
-        key = self.make_key(key, version=version)
-        self.validate_key(key)
-        return os.path.join(self._dir, ''.join(
-            [hashlib.md5(key.encode()).hexdigest(), self.cache_suffix]))
+        key = self.make_and_validate_key(key, version=version)
+        return os.path.join(
+            self._dir,
+            "".join(
+                [
+                    md5(key.encode(), usedforsecurity=False).hexdigest(),
+                    self.cache_suffix,
+                ]
+            ),
+        )
 
     def clear(self):
         """
@@ -154,5 +165,5 @@ class FileBasedCache(BaseCache):
         """
         return [
             os.path.join(self._dir, fname)
-            for fname in glob.glob1(self._dir, '*%s' % self.cache_suffix)
+            for fname in glob.glob1(self._dir, "*%s" % self.cache_suffix)
         ]
