@@ -1,7 +1,9 @@
 import fnmatch
 import os
+import shutil
+import subprocess
 from pathlib import Path
-from subprocess import PIPE, run
+from subprocess import run
 
 from django.apps import apps as installed_apps
 from django.utils.crypto import get_random_string
@@ -17,7 +19,7 @@ def popen_wrapper(args, stdout_encoding="utf-8"):
     Return stdout output, stderr output, and OS status code.
     """
     try:
-        p = run(args, stdout=PIPE, stderr=PIPE, close_fds=os.name != "nt")
+        p = run(args, capture_output=True, close_fds=os.name != "nt")
     except OSError as err:
         raise CommandError("Error executing %s" % args[0]) from err
     return (
@@ -153,3 +155,14 @@ def is_ignored_path(path, ignore_patterns):
         )
 
     return any(ignore(pattern) for pattern in normalize_path_patterns(ignore_patterns))
+
+
+def run_formatters(written_files):
+    """
+    Run the black formatter on the specified files.
+    """
+    if black_path := shutil.which("black"):
+        subprocess.run(
+            [black_path, "--fast", "--", *written_files],
+            capture_output=True,
+        )
