@@ -24,7 +24,7 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     supports_select_difference = False
     supports_slicing_ordering_in_compound = True
     supports_index_on_text_field = False
-    has_case_insensitive_like = False
+    supports_update_conflicts = True
     create_test_procedure_without_params_sql = """
         CREATE PROCEDURE test_procedure ()
         BEGIN
@@ -47,15 +47,37 @@ class DatabaseFeatures(BaseDatabaseFeatures):
 
     supports_order_by_nulls_modifier = False
     order_by_nulls_first = True
+    supports_logical_xor = True
+
+    @cached_property
+    def minimum_database_version(self):
+        if self.connection.mysql_is_mariadb:
+            return (10, 3)
+        else:
+            return (5, 7)
+
+    @cached_property
+    def bare_select_suffix(self):
+        if (
+            self.connection.mysql_is_mariadb and self.connection.mysql_version < (10, 4)
+        ) or (
+            not self.connection.mysql_is_mariadb
+            and self.connection.mysql_version < (8,)
+        ):
+            return " FROM DUAL"
+        return ""
 
     @cached_property
     def test_collations(self):
         charset = "utf8"
-        if self.connection.mysql_is_mariadb and self.connection.mysql_version >= (
-            10,
-            6,
+        if (
+            self.connection.mysql_is_mariadb
+            and self.connection.mysql_version >= (10, 6)
+        ) or (
+            not self.connection.mysql_is_mariadb
+            and self.connection.mysql_version >= (8, 0, 30)
         ):
-            # utf8 is an alias for utf8mb3 in MariaDB 10.6+.
+            # utf8 is an alias for utf8mb3 in MariaDB 10.6+ and MySQL 8.0.30+.
             charset = "utf8mb3"
         return {
             "ci": f"{charset}_general_ci",
@@ -86,6 +108,12 @@ class DatabaseFeatures(BaseDatabaseFeatures):
             "scalar value but it's not implemented (#25287).": {
                 "expressions.tests.FTimeDeltaTests.test_durationfield_multiply_divide",
             },
+            "UPDATE ... ORDER BY syntax on MySQL/MariaDB does not support ordering by"
+            "related fields.": {
+                "update.tests.AdvancedTests."
+                "test_update_ordered_by_inline_m2m_annotation",
+                "update.tests.AdvancedTests.test_update_ordered_by_m2m_annotation",
+            },
         }
         if "ONLY_FULL_GROUP_BY" in self.connection.sql_mode:
             skips.update(
@@ -146,6 +174,25 @@ class DatabaseFeatures(BaseDatabaseFeatures):
                     },
                 }
             )
+        if (
+            self.connection.mysql_is_mariadb and self.connection.mysql_version < (10, 4)
+        ) or (
+            not self.connection.mysql_is_mariadb
+            and self.connection.mysql_version < (8,)
+        ):
+            skips.update(
+                {
+                    "Parenthesized combined queries are not supported on MySQL < 8 and "
+                    "MariaDB < 10.4": {
+                        "queries.test_qs_combinators.QuerySetSetOperationTests."
+                        "test_union_in_subquery",
+                        "queries.test_qs_combinators.QuerySetSetOperationTests."
+                        "test_union_in_subquery_related_outerref",
+                        "queries.test_qs_combinators.QuerySetSetOperationTests."
+                        "test_union_in_with_ordering",
+                    }
+                }
+            )
         if not self.supports_explain_analyze:
             skips.update(
                 {
@@ -225,7 +272,7 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     @cached_property
     def supports_column_check_constraints(self):
         if self.connection.mysql_is_mariadb:
-            return self.connection.mysql_version >= (10, 2, 1)
+            return True
         return self.connection.mysql_version >= (8, 0, 16)
 
     supports_table_check_constraints = property(
@@ -236,11 +283,7 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     def can_introspect_check_constraints(self):
         if self.connection.mysql_is_mariadb:
             version = self.connection.mysql_version
-            return (version >= (10, 2, 22) and version < (10, 3)) or version >= (
-                10,
-                3,
-                10,
-            )
+            return version >= (10, 3, 10)
         return self.connection.mysql_version >= (8, 0, 16)
 
     @cached_property
@@ -252,7 +295,7 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     @cached_property
     def has_select_for_update_nowait(self):
         if self.connection.mysql_is_mariadb:
-            return self.connection.mysql_version >= (10, 3, 0)
+            return True
         return self.connection.mysql_version >= (8, 0, 1)
 
     @cached_property
@@ -290,6 +333,9 @@ class DatabaseFeatures(BaseDatabaseFeatures):
         """
         return self._mysql_storage_engine != "MyISAM"
 
+    uses_savepoints = property(operator.attrgetter("supports_transactions"))
+    can_release_savepoints = property(operator.attrgetter("supports_transactions"))
+
     @cached_property
     def ignores_table_name_case(self):
         return self.connection.mysql_server_data["lower_case_table_names"]
@@ -302,7 +348,7 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     @cached_property
     def supports_json_field(self):
         if self.connection.mysql_is_mariadb:
-            return self.connection.mysql_version >= (10, 2, 7)
+            return True
         return self.connection.mysql_version >= (5, 7, 8)
 
     @cached_property
@@ -313,14 +359,22 @@ class DatabaseFeatures(BaseDatabaseFeatures):
 
     @cached_property
     def supports_index_column_ordering(self):
-        return (
-            not self.connection.mysql_is_mariadb
-            and self.connection.mysql_version >= (8, 0, 1)
-        )
+        if self._mysql_storage_engine != "InnoDB":
+            return False
+        if self.connection.mysql_is_mariadb:
+            return self.connection.mysql_version >= (10, 8)
+        return self.connection.mysql_version >= (8, 0, 1)
 
     @cached_property
     def supports_expression_indexes(self):
         return (
             not self.connection.mysql_is_mariadb
+            and self._mysql_storage_engine != "MyISAM"
             and self.connection.mysql_version >= (8, 0, 13)
         )
+
+    @cached_property
+    def can_rename_index(self):
+        if self.connection.mysql_is_mariadb:
+            return self.connection.mysql_version >= (10, 5, 2)
+        return True
