@@ -226,7 +226,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
     def get_relations(self, cursor, table_name):
         """
         Return a dictionary of {field_name: (field_name_other_table, other_table)}
-        representing all relationships to the given table.
+        representing all foreign keys in the given table.
         """
         table_name = table_name.upper()
         cursor.execute(
@@ -248,26 +248,6 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
             for field_name, rel_table_name, rel_field_name in cursor.fetchall()
         }
 
-    def get_key_columns(self, cursor, table_name):
-        cursor.execute(
-            """
-            SELECT
-                ccol.column_name,
-                rcol.table_name AS referenced_table,
-                rcol.column_name AS referenced_column
-            FROM user_constraints c
-            JOIN user_cons_columns ccol
-              ON ccol.constraint_name = c.constraint_name
-            JOIN user_cons_columns rcol
-              ON rcol.constraint_name = c.r_constraint_name
-            WHERE c.table_name = %s AND c.constraint_type = 'R'""",
-            [table_name.upper()],
-        )
-        return [
-            tuple(self.identifier_converter(cell) for cell in row)
-            for row in cursor.fetchall()
-        ]
-
     def get_primary_key_column(self, cursor, table_name):
         cursor.execute(
             """
