@@ -6,7 +6,7 @@ from django.forms.utils import ErrorList, RenderableFormMixin
 from django.forms.widgets import CheckboxInput, HiddenInput, NumberInput
 from django.utils.functional import cached_property
 from django.utils.translation import gettext_lazy as _
-from django.utils.translation import ngettext
+from django.utils.translation import ngettext_lazy
 
 __all__ = ("BaseFormSet", "formset_factory", "all_valid")
 
@@ -32,19 +32,15 @@ class ManagementForm(Form):
     as well.
     """
 
-    def __init__(self, *args, **kwargs):
-        self.base_fields[TOTAL_FORM_COUNT] = IntegerField(widget=HiddenInput)
-        self.base_fields[INITIAL_FORM_COUNT] = IntegerField(widget=HiddenInput)
-        # MIN_NUM_FORM_COUNT and MAX_NUM_FORM_COUNT are output with the rest of
-        # the management form, but only for the convenience of client-side
-        # code. The POST value of them returned from the client is not checked.
-        self.base_fields[MIN_NUM_FORM_COUNT] = IntegerField(
-            required=False, widget=HiddenInput
-        )
-        self.base_fields[MAX_NUM_FORM_COUNT] = IntegerField(
-            required=False, widget=HiddenInput
-        )
-        super().__init__(*args, **kwargs)
+    template_name = "django/forms/div.html"  # RemovedInDjango50Warning.
+
+    TOTAL_FORMS = IntegerField(widget=HiddenInput)
+    INITIAL_FORMS = IntegerField(widget=HiddenInput)
+    # MIN_NUM_FORM_COUNT and MAX_NUM_FORM_COUNT are output with the rest of the
+    # management form, but only for the convenience of client-side code. The
+    # POST value of them returned from the client is not checked.
+    MIN_NUM_FORMS = IntegerField(required=False, widget=HiddenInput)
+    MAX_NUM_FORMS = IntegerField(required=False, widget=HiddenInput)
 
     def clean(self):
         cleaned_data = super().clean()
@@ -67,8 +63,19 @@ class BaseFormSet(RenderableFormMixin):
             "ManagementForm data is missing or has been tampered with. Missing fields: "
             "%(field_names)s. You may need to file a bug report if the issue persists."
         ),
+        "too_many_forms": ngettext_lazy(
+            "Please submit at most %(num)d form.",
+            "Please submit at most %(num)d forms.",
+            "num",
+        ),
+        "too_few_forms": ngettext_lazy(
+            "Please submit at least %(num)d form.",
+            "Please submit at least %(num)d forms.",
+            "num",
+        ),
     }
-    template_name = "django/forms/formsets/default.html"
+
+    template_name_div = "django/forms/formsets/div.html"
     template_name_p = "django/forms/formsets/p.html"
     template_name_table = "django/forms/formsets/table.html"
     template_name_ul = "django/forms/formsets/ul.html"
@@ -120,6 +127,22 @@ class BaseFormSet(RenderableFormMixin):
         """
         return True
 
+    def __repr__(self):
+        if self._errors is None:
+            is_valid = "Unknown"
+        else:
+            is_valid = (
+                self.is_bound
+                and not self._non_form_errors
+                and not any(form_errors for form_errors in self._errors)
+            )
+        return "<%s: bound=%s valid=%s total_forms=%s>" % (
+            self.__class__.__qualname__,
+            self.is_bound,
+            is_valid,
+            self.total_form_count(),
+        )
+
     @cached_property
     def management_form(self):
         """Return the ManagementForm instance for this FormSet."""
@@ -414,12 +437,7 @@ class BaseFormSet(RenderableFormMixin):
                 TOTAL_FORM_COUNT
             ] > self.absolute_max:
                 raise ValidationError(
-                    ngettext(
-                        "Please submit at most %d form.",
-                        "Please submit at most %d forms.",
-                        self.max_num,
-                    )
-                    % self.max_num,
+                    self.error_messages["too_many_forms"] % {"num": self.max_num},
                     code="too_many_forms",
                 )
             if (
@@ -430,12 +448,7 @@ class BaseFormSet(RenderableFormMixin):
                 < self.min_num
             ):
                 raise ValidationError(
-                    ngettext(
-                        "Please submit at least %d form.",
-                        "Please submit at least %d forms.",
-                        self.min_num,
-                    )
-                    % self.min_num,
+                    self.error_messages["too_few_forms"] % {"num": self.min_num},
                     code="too_few_forms",
                 )
             # Give self.clean() a chance to do cross-form validation.
@@ -507,6 +520,10 @@ class BaseFormSet(RenderableFormMixin):
         else:
             return self.empty_form.media
 
+    @property
+    def template_name(self):
+        return self.renderer.formset_template_name
+
     def get_context(self):
         return {"formset": self}
 
