@@ -80,7 +80,7 @@ def patch_cache_control(response, **kwargs):
     elif "public" in cc and "private" in kwargs:
         del cc["public"]
 
-    for (k, v) in kwargs.items():
+    for k, v in kwargs.items():
         directive = k.replace("_", "-")
         if directive == "no-cache":
             # no-cache supports multiple field names.
